"""M2Crypto PGP2.

Copyright (c) 1999-2003 Ng Pheng Siong. All rights reserved."""

CTB_TAG             = 128

CTB_PKE             = 1
CTB_SIGNATURE       = 2
CTB_MESSAGE_DIGETS  = 3
CTB_PRIVATE_KEY     = 5
CTB_PUBLIC_KEY      = 6
CTB_COMPRESSED_DATA = 8
CTB_CKE             = 9
CTB_LITERAL_DATA    = 11
CTB_TRUST           = 12
CTB_USERID          = 13
CTB_COMMENT         = 14


